#ifndef _CHUNKYIRCD_H_

#define _CHUNKYIRCD_H_

#include <time.h>
#include <logger.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "enpress.h"

/* define all the structures and sizes used in the code */
/* should be far cleaner */

/* Server-related defines */
#define SERVNAME_MAX 		63
#define SERVINFO_MAX		100
#define CHANNEL_MAX_LENGTH	64
#define TOPIC_LENGTH		300
#define CHANNEL_HASH_SIZE	500
#define BAN_LENGTH		100
#define NICK_LENGTH		20
#define HOST_LENGTH		63
#define USER_LENGTH		15
#define REAL_NAME_LENGTH	20
/* Flags */
#define SERV_ZIP		USR_ZIP
#define SERV_TS4		USR_TS4
#define SERV_ENCRYPT		USR_ENCRYPT

/* server structure */
struct _serverinfo {
	/* Host/Name/Info */
	char host[HOST_LENGTH + 1];
	char name[SERVNAME_MAX + 1];
	char info[SERVINFO_MAX + 1];
	/* File descriptor */
	int fd;
	/* Number of hops */
	int hops;
	/* Flags */
	int flags;
	/* Lastrecv */
	time_t lastrecv;
	/* Via */
	struct _serverinfo *via_ptr;
	/* TS Delta */
	time_t ts_delta;
	/* TS Version */
	int ts_version;
	/* Counters */
	long int client_count;
	int server_count;
	/* Used for one per server sending */
	int rand_no;
#ifdef USE_OPENSSL
	EncryptInfo ctx;
#endif
};

typedef struct _serverinfo *ServerInfo;

/* User modes on a channel */
#define CHANUSR_OP		1	/* Channel Operator */
#define CHANUSR_VOICE		2	/* Channel Voice */
#define CHANUSR_HALFOP		4	/* Channel Half-Operator */

/* small wrapper round client, for listing */
struct _user_wrap {
	/* next in list */
	struct _user_wrap *next;
	/* Actual pointer to client */
	struct _clientinfo *user_info;
	/* Channel Mode */
	int chan_mode;
};

typedef struct _user_wrap *ChannelMember;

/* Channel modes */
#define CMODE_TOPIC		1	/* Only chanops can change topic */
#define CMODE_INVONLY		2	/* Only invited peeps can join */
#define CMODE_MODERATED		4	/* Only VOICE/HALFOP/OP can talk */
#define CMODE_MEMONLYSEND	8	/* Only members of the channel can join */
#define CMODE_SECRET		16	/* Doesn't appear on LISTs or WHOIS */
#define CMODE_PRIVATE		32
#define CMODE_KEY 		64	/* Has a password (key) set */
#define CMODE_SERVREG		128	/* Is registered with chanserv */
#define CMODE_REGONLY		256	/* Only registered users can connect */

/* Channel structure */
struct _channel {
	char name[CHANNEL_MAX_LENGTH + 1];
	char topic[TOPIC_LENGTH + 1];
	/* Who set the topic */
	char topic_set[NICK_LENGTH + 1];
	/* Time the topic was set */
	time_t topic_ts;
	/* Time channel created */
	time_t ts;
	/* Start of user list */
	ChannelMember users_start;
	/* Channel mode */
	int mode;
	/* Ban list */
	char **ban_list;
	int ban_size;
	/* Channel key */
	char *key;
	/* Users on the channel */
	int user_count;
	/* User limit */
	int user_limit;
	/* Pointer to next in list */
	struct _channel *next;
};

typedef struct _channel *ChannelInfo;

/* Client defines */

/* Flags on a user */
#define USR_NICK		1	/* Have they sent a nick */
#define USR_USER		2	/* Have they sent a USER ? */
#define USR_RESOLVED		4	/* Has the resolver returned on them */
#define USR_PINGSENT		8	/* Have we pinged them ? */
#define USR_SERVER		16	/* Has the client sent a SERVER */
#define USR_CONNECT		32	/* Connection we're trying to use */
#define USR_SHOULDCONN		64	/* Should we connect when its resolved */
#define USR_SENTMYINFO		128	/* Did we send our SERVER to this client */
#define USR_SENTENCRYPT		256
#define USR_PASS		512	/* Did they send a successful password ? */
#define USR_TS4			1024	/* TS4 Capab announced */
#define USR_ZIP			2048
#define USR_ENCRYPT		4096
#define USR_SENTINITAUTH	8192

/* User modes */
#define MODE_AWAY		1	/* User is away */
#define MODE_WALLOPS		2	/* User wants to receive Wallops */
#define MODE_INVIS		4	/* Inivisible, doesn't show up on names */
#define MODE_REGIST		8	/* User is registered with nickserv */
#define MODE_IRCOP		16	/* User is an IRC Operator */
#define MODE_SERVNOT		32	/* User wants to receive Server notices */


/* Client structure */
struct _clientinfo {
	/* Pointers in trees */
	struct _clientinfo *left;
	struct _clientinfo *right;
	struct _clientinfo *parent;
	/* fd */
	int fd;
	/* Client mode */
	int mode;
	/* Flags */
	int flags;
	/* Last message time */
	time_t lastrecv;
	/* Flood protection timer */
	time_t flood_timer;
	/* Idle timer */
	time_t idle_timer;
	/* Info */
	char host[HOST_LENGTH + 1];
	char nick[NICK_LENGTH + 1];
	char user[USER_LENGTH + 1];
	char real_name[REAL_NAME_LENGTH + 1];
	char *away_msg;
	/* Channel list stuff */
	ChannelInfo *chan_list;
	int chanlist_no;
	int chan_count;
	/* Server client is connected to */
	ServerInfo server;
	/* Timestamp */
	time_t ts;
	/* Tag for stuff */
	int rand_no;
};

typedef struct _clientinfo *ClientInfo;

/*
* Temporary Structure
*/
struct tempinfo {
	char host[HOST_LENGTH + 1];
	char nick[NICK_LENGTH + 1];
	char user[USER_LENGTH + 1];
	char real_name[REAL_NAME_LENGTH + 1];
	char info[SERVINFO_MAX + 1];
	char name[SERVNAME_MAX + 1];
	int fd;
	int flags;
	int mode;
	int port;
	struct tempinfo *next;
	time_t lastrecv;
	/* Space for CONNECT stuff */
	struct in_addr connect_ip;
	char connect_host[HOST_LENGTH + 1];
#ifdef USE_OPENSSL
	EncryptInfo ctx;
#endif
};

typedef struct tempinfo *TempInfo;

#endif
