#ifndef _CONFIG_FILE_H_

#define _CONFIG_FILE_H_

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <assert.h>
#include <stdlib.h>
#include <unistd.h>
#include "utility.h"
#include "config.h"
#include "chunkyircd.h"
#include "server.h"
#include "modules.h"
#include <signal.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* related to getopt() */
extern char *optarg;
extern int optind, opterr, optopt;

#define MAX_OPS 10
#define MOTD_LINES_MAX 30
#define OP_NAME_LEN 20
#define OP_PASS_LEN 20
#define OP_HOST_LEN 63
#define PASS_LENGTH 20
#define USERMODES "iwsor"
#define CHANMODES "ntovblmkhsrR"
#define VERSION_TAG "2.8/monkeyircd-"

/* Runtime vars describing server state */
extern char server_name[SERVNAME_MAX + 1];
extern char server_info[SERVINFO_MAX + 1];
extern char curr_conf_file[200];
extern char load_date[40];
extern struct irc_op **olines;
extern int mass_kicks;
extern char **motd;
extern int max_chancount;
extern int ops;
extern int services;
extern long int channels;
extern long int temp_count;
/* Other stuff */
/* Mainly command line */
extern int debug;
extern int shouldfork;

/* Irc op structure in memory */
struct irc_op {
	char name[OP_NAME_LEN];
	char pass[OP_PASS_LEN];
	char hostmask[OP_HOST_LEN];
};

/* Cline, array is static to the config file routines */
struct cline {
	char serv_name[SERVNAME_MAX + 1];
	char host[HOST_LENGTH + 1];
	int port;
	char password[PASS_LENGTH + 1];
	unsigned char key[17];
};

struct nline {
	char serv_name[SERVNAME_MAX + 1];
	char host[HOST_LENGTH + 1];
	char password[PASS_LENGTH + 1];
	unsigned char key[17];
};

struct config_node {
	const char *name;
	void (*worker) (char *, char *, int[]);
};

/* Config file parsing functions */
int parse_config_file(char *, int[]);
/* Parse command line */
int parse_cmdline(int, char *[]);
/* CLINE/NLINE Stuff */
struct cline *get_cline(char *);
struct nline *get_nline(char *);
/* re-read config file */
void rehash(int);

#endif
