/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: ban.c,v 1.10.4.2 2002/04/13 17:08:28 lloydy Exp $
 *
 */
#include "ban.h"

/* Routines */
static int split_ban(struct baninfo *, const char *);
static int enlarge_ban_list(ChannelInfo);
static int shrink_ban_list(ChannelInfo);

/*
* Add a ban to a channel
*/
char *add_ban(ChannelInfo chan, const char *ban_string)
{
	int i;
	struct baninfo test;
	static char full_ban[100];

	if (!(ban_string) || !(split_ban(&test, ban_string)))
		return NULL;


	for (i = 0; i < chan->ban_size && chan->ban_list[i] != NULL; i++);

	if (i == chan->ban_size) {
		enlarge_ban_list(chan);
		i = chan->ban_size - 10;
	}

	/* Copy it in */
	ircd_sprintf(full_ban, 120, "%s!%s@%s", test.nick, test.user,
		     test.host);
	chan->ban_list[i] = string_alloc(strlen(full_ban));
	strcpy(chan->ban_list[i], full_ban);
	(chan->ban_list[i])[strlen(full_ban)] = '\0';


	if (chan->ban_list[i] == NULL) {
		return NULL;
	}

	/* Right, return me thinks */
	return full_ban;
}

/*
* Remove a ban from a channel
*/
int rem_ban(ChannelInfo chan, const char *ban_string)
{
	int i, j;

	for (i = 0; i < chan->ban_size && chan->ban_list[i] != NULL; i++) {

		if (lc_strncmp(chan->ban_list[i], ban_string, -1) == 0) {
			/* remove */
			free(chan->ban_list[i]);
			chan->ban_list[i] = NULL;

			/* Look for replacement */
			for (j = chan->ban_size - 1; j > i; j--) {
				/* Replace */
				if (chan->ban_list[j] != NULL) {
					chan->ban_list[i] =
					    chan->ban_list[j];
					chan->ban_list[j] = NULL;
					/* Check for size and return */
					shrink_ban_list(chan);
					return 1;
				}
			}
			/* Check for size reduction */
			shrink_ban_list(chan);
			/* We couldn't replace, no matter */
			return 1;
		}
	}

	return 0;
}

/*
* Check to see if a clients banned
*/
int check_ban(ChannelInfo chan, ClientInfo client)
{
	struct baninfo ban_info;
	int i;

	for (i = 0; i < chan->ban_size && chan->ban_list[i] != NULL; i++) {
		split_ban(&ban_info, chan->ban_list[i]);
		if (check_string(client->nick, (char *) ban_info.nick)) {
			if (check_string
			    (client->user, (char *) ban_info.user)) {
				if (check_string
				    (client->host,
				     (char *) ban_info.host)) {
					return 1;
				}
			}
		}

	}
	return 0;

}


static int split_ban(struct baninfo *ban_info, const char *ban_string)
{
	char buffer[150];
	char *nick_ptr = NULL, *user_ptr = NULL, *host_ptr = NULL;
	int i;

	strcpy(buffer, ban_string);

	buffer[strlen(ban_string)] = '\0';

	nick_ptr = buffer;

	for (i = 0; buffer[i] != '\0'; i++) {
		if (buffer[i] == '!') {
			user_ptr = &(buffer[i + 1]);
			buffer[i] = '\0';
		}
		if (buffer[i] == '@') {
			host_ptr = &(buffer[i + 1]);
			buffer[i] = '\0';
		}
	}

	if (nick_ptr) {
		strncpy((char *) ban_info->nick, (char *) nick_ptr,
			NICK_LENGTH);
	} else {
		strncpy((char *) ban_info->nick, "*", NICK_LENGTH);
	}
	if (user_ptr) {
		strncpy((char *) ban_info->user, (char *) user_ptr,
			USER_LENGTH);
	} else {
		strncpy((char *) ban_info->user, "*", USER_LENGTH);
	}
	if (host_ptr) {
		strncpy((char *) ban_info->host, (char *) host_ptr,
			HOST_LENGTH);
	} else {
		strncpy((char *) ban_info->host, "*", HOST_LENGTH);
	}

	return 1;
}


/*
 * Enlarge the ban list
 */
static int enlarge_ban_list(ChannelInfo chan)
{
	char **temp_ptr = NULL;
	int i;

	temp_ptr =
	    realloc(chan->ban_list,
		    (chan->ban_size + 10) * sizeof(char *));

	if (temp_ptr == NULL) {
		return 0;
	} else {
		chan->ban_list = temp_ptr;
		chan->ban_size += 10;

		/* NULL out newly allocated memory */
		for (i = chan->ban_size - 10; i < chan->ban_size; i++) {
			chan->ban_list[i] = NULL;
		}
		return 1;
	}
}

/*
 * Shrink the channel list
 */
static int shrink_ban_list(ChannelInfo chan)
{
	int i, non_null = 0;
	char **temp_ptr = NULL;

	/* Check that the area we're about to clean doesn't contain 
	 * any bans */
	for (i = chan->ban_size - 10; i < chan->ban_size && non_null == 0;
	     i++) {
		if (chan->ban_list[i]) {
			non_null = 1;
		}
	}

	if (non_null == 0 && chan->ban_size > 10) {
		temp_ptr =
		    realloc(chan->ban_list,
			    (chan->ban_size - 10) * sizeof(char *));

		if (temp_ptr == NULL) {
			return 0;
		} else {
			chan->ban_list = temp_ptr;
			chan->ban_size -= 10;
		}
	}

	return 1;
}
