/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_squit.c,v 1.10.4.14 2002/04/14 19:40:29 lloydy Exp $
 *
 */
#include "modules.h"

CMIRCD_MODULE(1.03, CMD_SERVER | CMD_CLIENT);

/* 
 * cmd_squit, available to opers to force a server connect, or
 * passed by a server to indicate another server has lost the connection
*/
int cmd_squit(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
	ServerInfo target = NULL;
	ServerInfo origin_serv = NULL;
	char buffer[BUF_SIZE];

	/* Check for IRC ops if its a client */
	if ((c_ptr) && !(c_ptr->mode & MODE_IRCOP)) {
		send_numeric(c_ptr, get_my_server(), 481,
			     ":Permission denied - You're no IRCOP");
		return 0;
	}

	/* Check the params */
	if (!(cmd->params[0]) || !(cmd->text)) {
		if (c_ptr) {
			/* Only reply if its local client */
			send_numeric(c_ptr, get_my_server(), 461,
				     "SQUIT :Not enough parameters");
		}
		return 0;
	}

	/* Check the target is valid */
	target = server_locate(cmd->params[0]);
	if (!target) {
		if (c_ptr) {
			ircd_sprintf(buffer, BUF_SIZE,
				     "%s :No such server", cmd->params[0]);
			send_numeric(c_ptr, get_my_server(), 402, buffer);
		}
		return 0;
	}

	/* Find the origin */
	if (cmd->origin) {
		origin_serv = server_locate(cmd->origin);
	}

	/* Inform everyone */
	/* Remove clients, servers we're connected via */
	ircd_sprintf(buffer, BUF_SIZE, "Received SQUIT for %s, Reason: %s",
		     target->name, cmd->text);
	clients_servnotice(buffer);

	/* Local SQUIT */
	/* Clients */
	if (origin_serv == NULL || origin_serv->ts_version >= 4) {
		/* Generate information */
		clients_netsplit(s_ptr, target, target->name);
		server_netsplit(s_ptr, target, cmd->text, target->name);
	}

	/* Decrement server count */
	target->via_ptr->server_count--;


	/* Do the work */
	if (target->via_ptr == get_my_server()) {
		/* Local server is the one to quit off */
		ircd_fdlist_clear_entry(target->fd);
		close(target->fd);
	}

	/* Carry on forwarding */
	ircd_sprintf(buffer, BUF_SIZE, "SQUIT %s :%s", target->name,
		     cmd->text);
	if (!(cmd->origin) || !(server_locate(cmd->origin))) {
		servers_relay(s_ptr, cmd->origin, buffer);
	} else {
		servers_relay(s_ptr, get_my_server()->name, buffer);
	}

	/* Remove from hash */
	server_hash_rem(target->name);
	server_free(target);

	return 1;
}
