/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_who.c,v 1.13.4.5 2002/04/14 19:34:00 lloydy Exp $
 *
 */
#include "modules.h"

CMIRCD_MODULE(1.02, CMD_CLIENT);

/* 
 * cmd_who.c - Part of Chunky Monkey IRCD - Copyright (c) 2000-2001 Chris Plant
 * Deals with WHO requests, should be able to match by channel or host, server
 * real name and nick if no channel matching the first mask are found
 * 
 * Status:
 * + 9th December - Full Implementation, not fully tested
 * + 8th December - Only Matches by channel (Lame)
 * 
 */

static int who_channel(ChannelInfo, ClientInfo);
static int who_common_channel(ClientInfo, ClientInfo);

/* 
 * cmd_who - deals with WHO requests 
 */
int cmd_who(ClientInfo sender, ServerInfo s_ptr, Cmd cmd)
{
	ChannelInfo chan = NULL;
	char buffer[BUF_SIZE];
	char *mask = NULL;

	if (!sender || s_ptr)
		return 0;

	if (!check_params(cmd, 1)) {
		send_numeric(sender, get_my_server(), 461,
			     "WHO :Not enough parameters");

		return 0;
	}

	/* Try and get a channel first */
	chan = locate_channel(cmd->params[0]);

	if (chan) {
		/* Deal with a channel */
		who_channel(chan, sender);
	} else {
		mask = cmd->params[0];
		if (mask[0] != '*' && mask[1] != '\0') {
			client_who_mask(mask, sender, NULL, 1);
		}
	}

	/* Send End of WHO */
	ircd_sprintf(buffer, BUF_SIZE, "%s :End of WHO List",
		     chan ? chan->name : cmd->params[0]);
	send_numeric(sender, get_my_server(), 315, buffer);


	return 1;
}

/*
 * If we get a channel, get the user list for that channel
 */
static int who_channel(ChannelInfo chan, ClientInfo c_ptr)
{
	ChannelMember on_chan = NULL;
	ChannelMember curr_wrap = NULL;
	char buffer[BUF_SIZE];

	/* Check on the channel */
	on_chan = channel_is_client_on(c_ptr, chan);
	curr_wrap = chan->users_start;

	while (curr_wrap) {
		if (who_common_channel(c_ptr, curr_wrap->user_info)) {
			ircd_sprintf(buffer, BUF_SIZE,
				     "%s %s %s %s %s %c%s :%0d %s",
				     chan->name,
				     curr_wrap->user_info->user,
				     curr_wrap->user_info->host,
				     curr_wrap->user_info->server->name,
				     curr_wrap->user_info->nick,
				     curr_wrap->user_info->
				     mode & MODE_AWAY ? 'G' : 'H',
				     curr_wrap->
				     chan_mode & CHANUSR_OP ? "@" :
				     curr_wrap->
				     chan_mode & CHANUSR_HALFOP ? "%" :
				     curr_wrap->
				     chan_mode & CHANUSR_VOICE ? "+" : "",
				     curr_wrap->user_info->server->hops,
				     curr_wrap->user_info->real_name);
			send_numeric(c_ptr, get_my_server(), 352, buffer);
		}
		curr_wrap = curr_wrap->next;

	}

	return 1;
}

/*
 * Try and find a common channel with the scanning user
 */
static int who_common_channel(ClientInfo sender, ClientInfo subject)
{
	int i;
	int j;

	/* Not invisible */
	if (!(subject->mode & MODE_INVIS))
		return 1;

	/* Loop thru channels on sender, and try to match with subject */
	for (i = 0;
	     sender->chan_list[i] != NULL && i < sender->chanlist_no;
	     i++) {
		for (j = 0;
		     subject->chan_list[j] != NULL
		     && j < subject->chanlist_no; j++) {
			if (subject->chan_list[j] == sender->chan_list[i])
				return 1;
		}
	}

	return 0;
}
